==> fingerprint-container-node-sdk-0.1.11.tgz: package/decoder.js <==
var util = require('util');
var EventEmitter = require('events');
var Buffer = require('buffer').Buffer;

var HEADER = {
    PACKET_LEN: 4,
    HEADER_LEN: 2,
    FLAG: 2,
    SERVICE: 4,
    TIME: 4,
    UNIQID: 4,
    ASKID: 4,
    CODE: 2,
    ROUTERS: 1,
    DSTMODE: 1,
    ROUTER_LIST_ITEM: 8,
    DSTPARAM: 8,
    BODY_PARSE: 0
};

var STATE = {
    PACKET_LEN: 'PACKET_LEN',
    HEADER_LEN: 'HEADER_LEN',
    FLAG: 'FLAG',
    SERVICE: 'SERVICE',
    TIME: 'TIME',
    UNIQID: 'UNIQID',
    ASKID: 'ASKID',
    CODE: 'CODE',
    ROUTERS: 'ROUTERS',
    DSTMODE: 'DSTMODE',
    ROUTER_LIST_ITEM: 'ROUTER_LIST_ITEM',
    DSTPARAM: 'DSTPARAM',
    BODY_PARSE: 'BODY_PARSE'
};

var OPTION_STATE = [
    STATE.ROUTER_LIST_ITEM,
    STATE.DSTPARAM,
    STATE.BODY_PARSE
];

function Decoder() {
    EventEmitter.call(this);
    this.state = STATE.PACKET_LEN;
    this.buffer = null;
    this.offset = 0;
    this.msg = {};
}

util.inherits(Decoder, EventEmitter);

Decoder.prototype.decode = function (buffer) {
    this.buffer = this.buffer ? Buffer.concat([this.buffer, buffer]) : new Buffer(buffer);

    while (true) {
        var bufferItem = null;
        var slicer = null;
        var headerLen = 0;
        var state = this.state;

        // 如果不是可选的字段，那么统一的进行处理（获取片段及长度判断）
        if (OPTION_STATE.indexOf(state) == -1) {
            headerLen = HEADER[state];
            slicer = this.buffer.slice(this.offset, this.offset + headerLen);
            if (slicer.length < headerLen) {
                break;
            }
        }

        switch (state) {
            case STATE.PACKET_LEN:
                this.msg.packetLen = slicer.readUInt32BE();
                this.state = STATE.HEADER_LEN;
                break;
            case STATE.HEADER_LEN:
                this.msg.headerLen = slicer.readUInt16BE();
                this.state = STATE.FLAG;
                break;
            case STATE.FLAG:
                this.msg.flag = slicer.readUInt16BE();
                this.state = STATE.SERVICE;
                break;
            case STATE.SERVICE:
                this.msg.service = slicer.readUInt32BE();
                this.state = STATE.TIME;
                break;
            case STATE.TIME:
                this.msg.time = slicer.readUInt32BE();
                this.state = STATE.UNIQID;
                break;
            case STATE.UNIQID:
                this.msg.uniqid = slicer.readUInt32BE();
                this.state = STATE.ASKID;
                break;
            case STATE.ASKID:
                this.msg.askId = slicer.readUInt32BE();
                this.state = STATE.CODE;
                break;
            case STATE.CODE:
                this.msg.code = slicer.readUInt16BE();
                this.state = STATE.ROUTERS;
                break;
            case STATE.ROUTERS:
                this.msg.routers = slicer.readUInt8();
                this.state = STATE.DSTMODE;
                break;
            case STATE.DSTMODE:
                this.msg.dstMode = slicer.readUInt8();
                this.state = STATE.ROUTER_LIST_ITEM;
                break;
            case STATE.ROUTER_LIST_ITEM:
                if (this.msg.routers == 0) {
                    this.state = STATE.DSTPARAM;
                    continue;
                }

                var routersList = [];
                var routers = this.msg.routers;
                headerLen = routers * HEADER.ROUTER_LIST_ITEM;
                slicer = this.buffer.slice(this.offset, this.offset + headerLen);
                if (slicer.length < headerLen) {
                    return;
                }

                for (var i = 0; i < routers; i++) {
                    var offset = i * HEADER.ROUTER_LIST_ITEM;
                    bufferItem = new Buffer(HEADER.ROUTER_LIST_ITEM);
                    slicer.copy(bufferItem, 0, offset, offset + HEADER.ROUTER_LIST_ITEM);
                    routersList.push(bufferItem);
                    this.offset += HEADER.ROUTER_LIST_ITEM;
                }

                this.msg.routersList = routersList;
                this.state = STATE.DSTPARAM;
                break;
            case STATE.DSTPARAM:
                if (this.msg.dstMode == 0) {
                    this.state = STATE.BODY_PARSE;
                    continue;
                }

                bufferItem = new Buffer(HEADER.DSTPARAM);
                this.buffer.copy(bufferItem, 0, this.offset, this.offset + HEADER.DSTPARAM);
                this.msg.dstParam = bufferItem;
                this.offset += HEADER.DSTPARAM;
                this.state = STATE.BODY_PARSE;
                break;
            case STATE.BODY_PARSE:
                var bodyLen = this.msg.packetLen - this.msg.headerLen - 2;
                var end = this.offset + bodyLen;
                var extraBuffer = this.buffer.slice(end);
                slicer = this.buffer.slice(this.offset, end);
                if (slicer.length < bodyLen) {
                    return;
                } else if (slicer.length == 0) {
                    this.emit('message', this.msg);
                } else {
                    try {
                        this.msg.body = JSON.parse(slicer.toString());
                        this.emit('message', this.msg);
                    } catch (e) {
                        // TODO log error
                        this.emit('error', e);
                    }
                }

                this._clean();
                this.buffer = extraBuffer;
                continue;
        }

        if (OPTION_STATE.indexOf(state) == -1) {
            this.offset += headerLen;
        }
    }
};

Decoder.prototype._clean = function () {
    this.buffer = null;
    this.offset = 0;
    this.state = STATE.PACKET_LEN;
    this.msg = {};
};

module.exports = Decoder;

==> fingerprint-container-node-sdk2-0.0.7.tgz: package/decoder.js <==
var util = require('util');
var EventEmitter = require('events');
var Buffer = require('buffer').Buffer;

var HEADER = {
    PACKET_LEN: 4,
    HEADER_LEN: 2,
    FLAG: 2,
    SERVICE: 4,
    TIME: 4,
    UNIQID: 4,
    ASKID: 4,
    CODE: 2,
    ROUTERS: 1,
    DSTMODE: 1,
    ROUTER_LIST_ITEM: 8,
    DSTPARAM: 8,
    BODY_PARSE: 0
};

var STATE = {
    PACKET_LEN: 'PACKET_LEN',
    HEADER_LEN: 'HEADER_LEN',
    FLAG: 'FLAG',
    SERVICE: 'SERVICE',
    TIME: 'TIME',
    UNIQID: 'UNIQID',
    ASKID: 'ASKID',
    CODE: 'CODE',
    ROUTERS: 'ROUTERS',
    DSTMODE: 'DSTMODE',
    ROUTER_LIST_ITEM: 'ROUTER_LIST_ITEM',
    DSTPARAM: 'DSTPARAM',
    BODY_PARSE: 'BODY_PARSE'
};

var OPTION_STATE = [
    STATE.ROUTER_LIST_ITEM,
    STATE.DSTPARAM,
    STATE.BODY_PARSE
];

function Decoder() {
    EventEmitter.call(this);
    this.state = STATE.PACKET_LEN;
    this.buffer = null;
    this.offset = 0;
    this.msg = {};
}

util.inherits(Decoder, EventEmitter);

Decoder.prototype.decode = function (buffer) {
    this.buffer = this.buffer ? Buffer.concat([this.buffer, buffer]) : new Buffer(buffer);

    while (true) {
        var bufferItem = null;
        var slicer = null;
        var headerLen = 0;
        var state = this.state;

        // 如果不是可选的字段，那么统一的进行处理（获取片段及长度判断）
        if (OPTION_STATE.indexOf(state) == -1) {
            headerLen = HEADER[state];
            slicer = this.buffer.slice(this.offset, this.offset + headerLen);
            if (slicer.length < headerLen) {
                break;
            }
        }

        switch (state) {
            case STATE.PACKET_LEN:
                this.msg.packetLen = slicer.readUInt32BE();
                this.state = STATE.HEADER_LEN;
                break;
            case STATE.HEADER_LEN:
                this.msg.headerLen = slicer.readUInt16BE();
                this.state = STATE.FLAG;
                break;
            case STATE.FLAG:
                this.msg.flag = slicer.readUInt16BE();
                this.state = STATE.SERVICE;
                break;
            case STATE.SERVICE:
                this.msg.service = slicer.readUInt32BE();
                this.state = STATE.TIME;
                break;
            case STATE.TIME:
                this.msg.time = slicer.readUInt32BE();
                this.state = STATE.UNIQID;
                break;
            case STATE.UNIQID:
                this.msg.uniqid = slicer.readUInt32BE();
                this.state = STATE.ASKID;
                break;
            case STATE.ASKID:
                this.msg.askId = slicer.readUInt32BE();
                this.state = STATE.CODE;
                break;
            case STATE.CODE:
                this.msg.code = slicer.readUInt16BE();
                this.state = STATE.ROUTERS;
                break;
            case STATE.ROUTERS:
                this.msg.routers = slicer.readUInt8();
                this.state = STATE.DSTMODE;
                break;
            case STATE.DSTMODE:
                this.msg.dstMode = slicer.readUInt8();
                this.state = STATE.ROUTER_LIST_ITEM;
                break;
            case STATE.ROUTER_LIST_ITEM:
                if (this.msg.routers == 0) {
                    this.state = STATE.DSTPARAM;
                    continue;
                }

                var routersList = [];
                var routers = this.msg.routers;
                headerLen = routers * HEADER.ROUTER_LIST_ITEM;
                slicer = this.buffer.slice(this.offset, this.offset + headerLen);
                if (slicer.length < headerLen) {
                    return;
                }

                for (var i = 0; i < routers; i++) {
                    var offset = i * HEADER.ROUTER_LIST_ITEM;
                    bufferItem = new Buffer(HEADER.ROUTER_LIST_ITEM);
                    slicer.copy(bufferItem, 0, offset, offset + HEADER.ROUTER_LIST_ITEM);
                    routersList.push(bufferItem);
                    this.offset += HEADER.ROUTER_LIST_ITEM;
                }

                this.msg.routersList = routersList;
                this.state = STATE.DSTPARAM;
                break;
            case STATE.DSTPARAM:
                if (this.msg.dstMode == 0) {
                    this.state = STATE.BODY_PARSE;
                    continue;
                }

                bufferItem = new Buffer(HEADER.DSTPARAM);
                this.buffer.copy(bufferItem, 0, this.offset, this.offset + HEADER.DSTPARAM);
                this.msg.dstParam = bufferItem;
                this.offset += HEADER.DSTPARAM;
                this.state = STATE.BODY_PARSE;
                break;
            case STATE.BODY_PARSE:
                var bodyLen = this.msg.packetLen - this.msg.headerLen - 2;
                var end = this.offset + bodyLen;
                var extraBuffer = this.buffer.slice(end);
                slicer = this.buffer.slice(this.offset, end);
                if (slicer.length < bodyLen) {
                    return;
                } else if (slicer.length == 0) {
                    this.emit('message', this.msg);
                } else {
                    try {
                        this.msg.body = slicer;
                        this.emit('message', this.msg);
                    } catch (e) {
                        // TODO log error
                        this.emit('error', e);
                    }
                }

                this._clean();
                this.buffer = extraBuffer;
                continue;
        }

        if (OPTION_STATE.indexOf(state) == -1) {
            this.offset += headerLen;
        }
    }
};

Decoder.prototype._clean = function () {
    this.buffer = null;
    this.offset = 0;
    this.state = STATE.PACKET_LEN;
    this.msg = {};
};

module.exports = Decoder;
